/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK 2020                               **
**                                                                        **
**                             UDP example                                **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2020.11.26                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include <LPC23xx.H>
#include "system.h"
#include "types.h"
#include "systime.h"
#include "irqvectors.h"
#include "serialconst.h"
#include "uart0.h"
#include "strings.h"
#include "emacdriver.h"
#include "ipservice.h"
#include "udpservice.h"
#include "trace.h"

#define IRQCallCntLimit                 500

typedef struct {
                 EMACInstanceRecT *          EMACLayer ;
                 IPLayerInstanceRecT *       IPLayer ;
                 PtrUDPLayerInstanceRecT     UDPLayer ;
               } UDPExampleInstanceRecT ;
typedef UDPExampleInstanceRecT * PtrUDPExampleInstanceRecT ;

static UDPExampleInstanceRecT UDPDemoInstance ; ;
static ULONG IRQCallCounter ;

#define MachineIPAddress        0xC0A80037
//                              ^^^ 192.168.0.55
#define MachineIPSubnet         0xFFFFFF00
//                              ^^^ 255.255.255.0
#define MachineGWIPAddress      0xC0A800FE
//                              ^^^ 192.168.0.254
#define UDPPortNumber           20001


UCHAR MachineMACAddr [ 6 ]     = { 0x08 , 0x14 , 0x16 , 0x22 , 0x28 , 0x2D } ;


static __irq void T1_IRQHandler ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter ++ ;
  if ( IRQCallCounter >= IRQCallCntLimit )
  {
    IRQCallCounter = 0 ;
  } /* if */ ;
  T1IR = 1 ;
  VICVectAddr = 0 ;
} /* T1_IRQHandler */


static void HardwareInit ( void ) 
{
  /*-------------------------------------------------------------------------*/
  PINSEL10 = 0 ;
  T1MR0 = ( PCLK / 1000 ) - 1 ;
  T1MCR = 3 ;
  T1TCR = 1 ;
  VICVectAddr5 = ( unsigned long ) T1_IRQHandler ;
  VICVectCntl5 = 15 ;
  VICIntEnable = ( 1 << TIMER1_Interrupt ) ;
  UART0InitEnvir ( ) ;
} /* HardwareInit */


static void StartNet ( PtrUDPExampleInstanceRecT Instance )
{
  EtherSpeedMode EthMode ;
  /*-------------------------------------------------------------------------*/
  EthMode = Mode_AutoNeg ;
//  EthMode = Mode_10MBIT ;
//  EthMode = Mode_100MBIT ;
  Instance -> EMACLayer = InitPHYLayerInstance ( MachineMACAddr , EthMode ) ;
  Instance -> IPLayer = InitIPLayerInstance ( Instance -> EMACLayer ,
                                              ( ULONG ) MachineIPAddress ,
                                              ( ULONG ) MachineIPSubnet ) ;
  Instance -> IPLayer -> MyDefaultGWIPAddress = ( ULONG ) MachineGWIPAddress ;
  Instance -> UDPLayer = StandardAddUDPInstance ( Instance -> IPLayer ) ;
} /* StartNet */


UCHAR Hello1 [ ]      = "\r\nHello: serwer UDP\r\n" ;
UCHAR Hello2 [ ]      = "\r\nMoj adres IP=" ;
UCHAR Hello3 [ ]      = "\r\nMaska podsieci=" ;
UCHAR Hello4 [ ]      = "\r\nAdres bramy domyslnej=" ;
UCHAR Hello6 [ ]      = "\r\nAdres MAC=" ;
UCHAR Hello7 [ ]      = "\r\n************************\r\n" ;
#define UDPPortNumber           20001
extern void TraceMAC ( UCHAR * /* MACAddr */ ) ;

extern void TraceIP ( ULONG /* IPAddress */ ) ;

static void HelloMessage ( void )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( Hello1 ) ;
  UART0SendString ( Hello2 ) ;
  TraceIP ( MachineIPAddress ) ;
  UART0SendString ( Hello3 ) ;
  TraceIP ( MachineIPSubnet ) ;
  UART0SendString ( Hello4 ) ;
  TraceIP ( MachineGWIPAddress ) ;
  UART0SendString ( Hello6 ) ;
  TraceMAC ( MachineMACAddr ) ;
  UART0SendString ( Hello7 ) ;
} /* HelloMessage */


static void SoftwareInit ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter = 0 ;
  UDPDemoInstance . EMACLayer = NIL ;
  UDPDemoInstance . IPLayer = NIL ;
  UDPDemoInstance . UDPLayer = NIL ;
} /* SoftwareInit */


UCHAR NoUDPData [ ]        = "Pusty pakiet UDP" ;
UCHAR SrcInfo01 [ ]        = "\r\nDane od IP=" ;
UCHAR SrcInfo02 [ ]        = " port=" ;
UCHAR SrcInfo03 [ ]        = " do IP=" ;
UCHAR SrcInfo04 [ ]        = " tekst=" ;

static void UDPStreamService ( PtrUDPSocketDescriptionRecT UDPSocket ,
                               ULONG                       SenderIPAddress ,
                               USHORT                      SenderPortNo ,
                               ULONG                       DestIPAddress ,
                               USHORT                      DestPortNo ,
                               void *                      UDPDataBuffer ,
                               USHORT                      UDPDataSize )
{
  UCHAR * ChPtr ;
  USHORT Loop ;
  UCHAR Number [ 20 ] ;
  /*-------------------------------------------------------------------------*/
  UART0SendString ( SrcInfo01 ) ;
  IPToString ( Number , SenderIPAddress ) ;
  UART0SendString ( Number ) ;
  UART0SendString ( SrcInfo02 ) ;
  WordToStr ( Number , 5 , ' ' , SenderPortNo ) ;
  UART0SendString ( BegOfString ( Number ) ) ;
  UART0SendString ( SrcInfo03 ) ;
  IPToString ( Number , DestIPAddress ) ;
  UART0SendString ( Number ) ;
  UART0SendString ( SrcInfo02 ) ;
  WordToStr ( Number , 5 , ' ' , DestPortNo ) ;
  UART0SendString ( BegOfString ( Number ) ) ;
  UART0SendString ( SrcInfo04 ) ;
  ChPtr = ( UCHAR * ) UDPDataBuffer ;
  if ( UDPDataSize == 0 )
  {
    UART0SendString ( NoUDPData ) ;
  } /* if ... */
  else
  {
    for ( Loop = 0 ; Loop < UDPDataSize ; Loop ++ )
    {
      UART0Send ( * ChPtr ) ;
      ChPtr ++ ;
    } /* for */ ;
  } /* if ... else */ ;
  UART0Send ( '\r' ) ;
  UART0Send ( '\n' ) ;
} /* UDPStreamService */


static void Halt ( UCHAR * Message )
{
  /*-------------------------------------------------------------------------*/
  UART0SendString ( Message ) ;
  for ( ; ; )
  {
    NOP ( ) ;
  } /* loop */ ;
} /* Halt */


UCHAR Error [ ]          = "\r\nProblem z gniazdkiem\r\n" ;

int main ( void )
{
  PtrUDPSocketDescriptionRecT LockUDPSocket ;
  USHORT BindStatus ;
  /*-------------------------------------------------------------------------*/
  SysInit ( ) ;
  InitSysTime ( ) ;
  HardwareInit ( )  ;
  SoftwareInit ( ) ;
  UART0HardwInit( FOSC , N81Mode , 9600 ) ;
  HelloMessage ( ) ;
  StartNet ( & UDPDemoInstance ) ;
  LockUDPSocket =  CreateUDPSocket ( UDPDemoInstance . UDPLayer ) ;
  BindStatus = BindUDPSocket ( LockUDPSocket , UDPPortNumber ) ;
  if ( BindStatus != UDP_BindPortOK )
  {
    Halt ( Error ) ;
  } /* if */ ;
  SetUDPListen ( LockUDPSocket , UDPStreamService ) ;
  for ( ; ; )
  {
    SysTimerPool ( ) ;
    UDPDemoInstance . IPLayer -> IPLayerPoolService ( UDPDemoInstance . IPLayer ) ;
  } /* loop */ ;
} /* main */

